@@ -13,7 +13,7 @@ use request::*;
 use anyhow::Result;
 use std::collections::HashMap;
 use std::sync::Arc;
-use tokio::sync::Mutex;
+use tokio::sync::{watch, Mutex};
 use tokio::time::{Duration, Instant};
 use util::Conn;
 
@@ -25,6 +25,7 @@ pub struct Server {
     realm: String,
     channel_bind_timeout: Duration,
     pub(crate) nonces: Arc<Mutex<HashMap<String, Instant>>>,
+    shutdown_tx: Option<watch::Sender<bool>>,
 }
 
 impl Server {
@@ -32,11 +33,14 @@ impl Server {
     pub async fn new(config: ServerConfig) -> Result<Self> {
         config.validate()?;
 
+        let (shutdown_tx, shutdown_rx) = watch::channel(false);
+
         let mut s = Server {
             auth_handler: config.auth_handler,
             realm: config.realm,
             channel_bind_timeout: config.channel_bind_timeout,
             nonces: Arc::new(Mutex::new(HashMap::new())),
+            shutdown_tx: Some(shutdown_tx),
         };
 
         if s.channel_bind_timeout == Duration::from_secs(0) {
@@ -48,6 +52,7 @@ impl Server {
             let auth_handler = Arc::clone(&s.auth_handler);
             let realm = s.realm.clone();
             let channel_bind_timeout = s.channel_bind_timeout;
+            let shutdown_rx = shutdown_rx.clone();
 
             tokio::spawn(async move {
                 let allocation_manager = Arc::new(Manager::new(ManagerConfig {
@@ -61,6 +66,7 @@ impl Server {
                     auth_handler,
                     realm,
                     channel_bind_timeout,
+                    shutdown_rx,
                 )
                 .await;
             });
@@ -76,16 +82,29 @@ impl Server {
         auth_handler: Arc<Box<dyn AuthHandler + Send + Sync>>,
         realm: String,
         channel_bind_timeout: Duration,
+        mut shutdown_rx: watch::Receiver<bool>,
     ) {
         let mut buf = vec![0u8; INBOUND_MTU];
 
         loop {
-            //TODO: gracefully exit loop
-            let (n, addr) = match conn.recv_from(&mut buf).await {
-                Ok((n, addr)) => (n, addr),
-                Err(err) => {
-                    log::debug!("exit read loop on error: {}", err);
-                    break;
+            let (n, addr) = tokio::select! {
+                v = conn.recv_from(&mut buf) => {
+                    match v {
+                        Ok(v) => v,
+                        Err(err) => {
+                            log::debug!("exit read loop on error: {}", err);
+                            break;
+                        }
+                    }
+                },
+                did_change = shutdown_rx.changed() => {
+                    if did_change.is_err() || *shutdown_rx.borrow() {
+                        // if did_change.is_err, sender was dropped, or if
+                        // bool is set to true, that means we're shutting down.
+                        break
+                    } else {
+                        continue;
+                    }
                 }
             };
 
@@ -106,10 +125,18 @@ impl Server {
         }
 
         let _ = allocation_manager.close().await;
+        let _ = conn.close().await;
     }
 
     // Close stops the TURN Server. It cleans up any associated state and closes all connections it is managing
-    pub fn close(&self) -> Result<()> {
+    pub async fn close(&mut self) -> Result<()> {
+        if let Some(tx) = self.shutdown_tx.take() {
+            // errors if there are no receivers, but that's irrelevant.
+            let _ = tx.send(true);
+            // wait for all receivers to drop/close.
+            let _ = tx.closed().await;
+        }
+
         Ok(())
     }
 }
